_check_port() {
	ipv_arg=$1
	ports=$2
	hosts=$3
	state=$4
	case "$ipv_arg" in
		ipv4)ipv=-4;;
		ipv6)ipv=-6;;
		*)ipv="";;
	esac
	case "$state" in
		open)grep_opt="grep -v";;
		*)grep_opt="grep";;
	esac
	tmpres=$(mktemp)

	nmap $ipv -oG - -n -Pn -p $ports $hosts >$tmpres

	if [ -n "$(cat $tmpres |grep "Ports: " |$grep_opt "/open/")" ]
	then
		fail "Port not $state '$hosts':$ports $ipv_arg"
		if [ "$verbose" -gt 1 ]
		then
			cat $tmpres |grep "Ports: " |$grep_opt "/open/" | while read line
			do
				logdate ERR "$line" 1
			done
		fi
	else
		success "Port $state '$hosts':$ports $ipv_arg"
	fi
	rm $tmpres
}

check_port_open()
{
	_check_ports_state "$1" "$2" "$3" "$4" "10" "open"
}

check_port_close()
{
	_check_ports_state "$1" "$2" "$3" "$4" "50" "close"
}

_fw_progress() {
	sz=$1
	timeout=$2
	if [ "$verbose" -gt 1 ]
	then
		cat | pv -l -s $sz -i $timeout -apte
	else
		cur=0
		printf "%4d/%4d" "$cur" "$sz" >&2
		while read line
		do
			printf "\b\b\b\b\b\b\b\b\b%4d/%4d" "$cur" "$sz" >&2
			cur="$(expr $cur + 1)"
		done
		echo -n "\b\b\b\b\b\b\b\b\b         \b\b\b\b\b\b\b\b\b" >&2
	fi
}

_check_ports_state() {
	ipv_arg=$1
	ports_arg=$2
	host=$3
	timeout=$4
	nprocs=$5
	state=$6
	case "$ipv_arg" in
		ipv4)ipv=-4;;
		ipv6)ipv=-6;;
		*)ipv="";;
	esac
	if [ -z "$timeout" ]
	then
		timeout=1
	fi
	if [ -z "$nprocs" ]
	then
		nprocs=10
	fi

	tmp_scan=$(mktemp -t port_scan_XXXXXXX.sh)
	echo "nc $ipv -z -w $timeout $host \$1 && echo \"\$1 open\"||echo \"\$1 close\"" > $tmp_scan
	chmod +x $tmp_scan

	ports=$(echo $ports_arg |tr -s ',' ' '| tr -s " " "\n" | while read port;
		do
			if echo $port | grep '-' >/dev/null
			then
				echo $(seq $(echo $port |tr -s '-' ' '))
			else
				echo $port
			fi
		done)

	ports_count=$(echo $ports | tr -s ' ' "\n" |wc -l)

	if [ "$verbose" -gt 1 ]
	then
		logdate INFO "$tc_name: Checking $ports_count ports ($ports_arg) are $state on $host $ipv_arg"
	fi

	#failures=$(echo $ports | tr -s ' ' "\n" | xargs -I '{}' -P $nprocs $tmp_scan '{}'| pv -l -s $ports_count -i $timeout -apte | grep -v "$state" | cut -d " " -f1)
	failures=$(echo $ports | tr -s ' ' "\n" | xargs -I '{}' -P $nprocs $tmp_scan '{}'| _fw_progress $ports_count $timeout | grep -v "$state" | cut -d " " -f1)

	if [ -n "$failures" ]
	then
		fail "Port not $state : $failures"
	else
		success "All ports $state : $ports_arg"
	fi

	rm $tmp_scan
}
